require 'test_helper'

describe Gauss do
  before do
    reset_gauss

    class T
      include Gauss
      include Tester
    end

    Gauss::mode = :local
  end

  after do
    Object.send(:remove_const, :T) rescue nil
  end

  it 'asserts true' do
    true.must_equal true
  end
  
  it 'uses rails app name as prefix' do
    class Gauss::Rails
      def self.application
        App.new
      end

      class App
        def self.parent_name
          return "test_app"
        end
      end
    end

    class T
      include Gauss
    end
    
    T.new.prefix.must_equal "test_app"

    # cleanup
    Gauss.send(:remove_const, :Rails)
  end
  
  it 'lets you override the prefix' do
    Gauss::benchmark_prefix = "test_prefix"
    Gauss::prefix.must_equal "test_prefix"
  end

  it 'executes code in the given block' do
    class TestException < StandardError; end
    
    class T
      def test
        benchmark {
          raise TestException
        }
      end
    end

    assert_raises(TestException) {
      T.new.test
    }
  end

  it 'outputs timing in local mode' do
    class T
      def test
        benchmark {
          1+1
        }
      end
    end

    out, _ = capture_io { T.new.test }
    assert_match /t\.test/, out
  end

  it 'uses optional suffix for the name' do
    class T
      def test
        benchmark("test_suffix") {
          1+1
        }
      end
    end

    out, _ = capture_io { T.new.test }
    assert_match /t\.test.*test_suffix/, out
  end

  it 'doesnt output anything if not in localmode' do
    Gauss::mode = :non_local
    class T
      def test
        benchmark("test_suffix") {
          1+1
        }
      end
    end

    out, _ = capture_io { T.new.test }
    out.must_equal ""
  end

  it 'calls metrics.time when not in localmode' do
    class O
      def time(_, &__)
        "test_worked"
      end
    end

    Gauss::mode = :non_local
    Gauss::metrics = O.new
    class T
      def test
        benchmark {
          1+1
        }
      end
    end
    
    T.new.test.must_equal "test_worked"
    Object.send(:remove_const, :O) rescue nil    
  end

  it 'uses statsd with envvars by default' do
    old_server = ENV["STATSD_SERVER"]
    old_port = ENV["STATSD_SERVER_PORT"]
    ENV["STATSD_SERVER"] = "test_server"
    ENV["STATSD_SERVER_PORT"] = "test_port"

    reset_gauss
    class T
      include Gauss
    end

    begin
      Gauss::metrics.host.must_equal "test_server"
      Gauss::metrics.port.must_equal "test_port"
    ensure
      ENV["STATSD_SERVER"] = old_server
      ENV["STATSD_SERVER_PORT"] = old_port
    end
  end
end
